@@ -1,22 +1,33 @@
 #
 # = bio.rb - Loading all BioRuby modules
 #
-# Copyright::	Copyright (C) 2001-2006
+# Copyright::	Copyright (C) 2001-2007
 #		Toshiaki Katayama <k@bioruby.org>
 # License::	The Ruby License
 #
-# $Id: bio.rb,v 1.86 2007/07/16 12:26:28 ngoto Exp $
+# $Id:$
 #
 
 module Bio
 
-  BIORUBY_VERSION = [1, 1, 0].extend(Comparable)
+  autoload :BIORUBY_VERSION, 'bio/version'
+  autoload :BIORUBY_EXTRA_VERSION, 'bio/version'
+  autoload :BIORUBY_VERSION_ID, 'bio/version'
 
   ### Basic data types
 
   ## Sequence
 
   autoload :Sequence,       'bio/sequence'
+  ## below are described in bio/sequence.rb
+  #class Sequence
+  #  autoload :Common,  'bio/sequence/common'
+  #  autoload :NA,      'bio/sequence/na'
+  #  autoload :AA,      'bio/sequence/aa'
+  #  autoload :Generic, 'bio/sequence/generic'
+  #  autoload :Format,  'bio/sequence/format'
+  #  autoload :Adapter, 'bio/sequence/adapter'
+  #end
 
   ## Locations/Location
 
@@ -26,12 +37,12 @@ module Bio
   ## Features/Feature
 
   autoload :Feature,        'bio/feature'
-  autoload :Features,       'bio/feature'
+  autoload :Features,       'bio/compat/features'
 
   ## References/Reference
 
   autoload :Reference,      'bio/reference'
-  autoload :References,     'bio/reference'
+  autoload :References,     'bio/compat/references'
 
   ## Pathway/Relation
 
@@ -102,8 +113,9 @@ module Bio
   ## other formats
 
   autoload :FastaFormat,    'bio/db/fasta'
-  autoload :FastaNumericFormat, 'bio/db/fasta' # change to FastaFormat::Numeric ?
-  autoload :FastaDefline,       'bio/db/fasta' # change to FastaFormat::Defline
+  autoload :FastaNumericFormat, 'bio/db/fasta/qual' # change to FastaFormat::Numeric ?
+  autoload :FastaDefline,       'bio/db/fasta/defline' # change to FastaFormat::Defline ?
+  autoload :Fastq,          'bio/db/fastq'
   autoload :GFF,            'bio/db/gff'
   autoload :AAindex,        'bio/db/aaindex'
   autoload :AAindex1,       'bio/db/aaindex' # change to AAindex::AAindex1 ?
@@ -119,10 +131,22 @@ module Bio
   autoload :REBASE,         'bio/db/rebase'
   autoload :SOFT,           'bio/db/soft'
   autoload :Lasergene,      'bio/db/lasergene'
+  autoload :SangerChromatogram, 'bio/db/sanger_chromatogram/chromatogram'
+  autoload :Scf,                'bio/db/sanger_chromatogram/scf'
+  autoload :Abif,               'bio/db/sanger_chromatogram/abif'
 
   autoload :Newick,         'bio/db/newick'
   autoload :Nexus,          'bio/db/nexus'
 
+  autoload :PhyloXML,       'bio/db/phyloxml/phyloxml_elements'
+  # Bio::Taxonomy will be moved to other file
+  autoload :Taxonomy,       'bio/db/phyloxml/phyloxml_elements'
+  ## below are described in bio/db/phyloxml/phyloxml_elements.rb
+  #module PhyloXML
+  #  autoload :Parser,       'bio/db/phyloxml/phyloxml_parser'
+  #  autoload :Writer,       'bio/db/phyloxml/phyloxml_writer'
+  #end
+
   ### IO interface modules
 
   autoload :Registry,       'bio/io/registry'
@@ -144,6 +168,7 @@ module Bio
   autoload :DBGET,          'bio/io/dbget'
 
   autoload :Ensembl,        'bio/io/ensembl'
+  autoload :Hinv,           'bio/io/hinv'
 
   ## below are described in bio/appl/blast.rb
   #class Blast
@@ -167,10 +192,14 @@ module Bio
     autoload :SOAP,         'bio/io/ebisoap'
   end
 
-  class NCBI
-    autoload :SOAP,       'bio/io/ncbisoap'
-  end
+  autoload :NCBI,         'bio/io/ncbirest'
+  ## below are described in bio/io/ncbirest.rb
+  #class NCBI
+  #  autoload :SOAP,       'bio/io/ncbisoap'
+  #  autoload :REST,       'bio/io/ncbirest'
+  #end
 
+  autoload :TogoWS,       'bio/io/togows'
 
   ### Applications
 
@@ -188,6 +217,9 @@ module Bio
   #  autoload :Default,      'bio/appl/blast/format0'
   #  autoload :WU,           'bio/appl/blast/wublast'
   #  autoload :Bl2seq,       'bio/appl/bl2seq/report'
+  #  autoload :RPSBlast,     'bio/appl/blast/rpsblast'
+  #  autoload :NCBIOptions,  'bio/appl/blast/ncbioptions'
+  #  autoload :Remote,       'bio/appl/blast/remote'
   #end
 
   autoload :HMMER,          'bio/appl/hmmer'
@@ -251,6 +283,14 @@ module Bio
 
   autoload :Iprscan,        'bio/appl/iprscan/report'
 
+  autoload :PAML,           'bio/appl/paml/common'
+  ## below are described in bio/appl/paml/common.rb
+  # module PAML
+  #   autoload :Codeml,         'bio/appl/paml/codeml'
+  #   autoload :Baseml,         'bio/appl/paml/baseml'
+  #   autoload :Yn00,           'bio/appl/paml/yn00'
+  # end
+
   ### Utilities
 
   autoload :SiRNA,          'bio/util/sirna'
