@@ -2,25 +2,9 @@
 # = bio/appl/sim4.rb - sim4 wrapper class
 #
 # Copyright:: Copyright (C) 2004 GOTO Naohisa <ng@bioruby.org>
-# License::   LGPL
+# License::   The Ruby License
 #
-#--
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#++
-#
-#  $Id: sim4.rb,v 1.5 2005/12/18 15:58:40 k Exp $
+#  $Id: sim4.rb,v 1.10 2007/04/05 23:35:39 trevor Exp $
 #
 # The sim4 execution wrapper class.
 #
@@ -31,8 +15,8 @@
 #   http://www.genome.org/cgi/content/abstract/8/9/967
 #
 
-require 'open3'
 require 'tempfile'
+require 'bio/command'
 
 module Bio
 
@@ -45,14 +29,13 @@ module Bio
     # [+program+]  Program name. Usually 'sim4' in UNIX.
     # [+database+] Default file name of database('seq2').
     # [+option+]   Options (array of strings).
-    def initialize(program = 'sim4', database = nil, option = [])
+    def initialize(program = 'sim4', database = nil, opt = [])
       @program = program
-      @option = option
+      @options = opt
       @database = database #seq2
       @command = nil
       @output = nil
       @report = nil
-      @log = nil
     end
 
     # default file name of database('seq2')
@@ -62,13 +45,27 @@ module Bio
     attr_reader :program
 
     # options
-    attr_reader :option
+    attr_accessor :options
+
+    # option is deprecated. Instead, please use options.
+    def option
+      warn "option is deprecated. Please use options."
+      options
+    end
 
     # last command-line strings executed by the object
     attr_reader :command
 
+    #---
     # last messages of program reported to the STDERR
-    attr_reader :log
+    #attr_reader :log
+    #+++
+
+    #log is deprecated (no replacement) and returns empty string.
+    def log
+      warn "log is deprecated (no replacement) and returns empty string."
+      ''
+    end
 
     # last result text (String)
     attr_reader :output
@@ -110,20 +107,13 @@ module Bio
     # <tt>filename1</tt> and <tt>filename2</tt> should be file name strings.
     # If <tt>filename2</tt> is not specified, using <tt>self.database</tt>.
     def exec_local(filename1, filename2 = nil)
-      @command = [ @program, filename1, (filename2 or @database), *@option ]
+      @command = [ @program, filename1, (filename2 or @database), *@options ]
       @output = nil
-      @log = nil
       @report = nil
-      Open3.popen3(*@command) do |din, dout, derr|
-        din.close
-        derr.sync = true
-        t = Thread.start { @log = derr.read }
-        begin
-          @output = dout.read
-          @report = Bio::Sim4::Report.new(@output)
-        ensure
-          t.join
-        end
+      Bio::Command.call_command(@command) do |io|
+        io.close_write
+        @output = io.read
+        @report = Bio::Sim4::Report.new(@output)
       end
       @report
     end
