@@ -6,7 +6,7 @@
 #             Alex Gutteridge <alexg@ebi.ac.uk>
 # License::   The Ruby License
 #
-#  $Id: pdb.rb,v 1.23 2007/07/10 10:44:46 ngoto Exp $
+#  $Id: pdb.rb,v 1.28 2008/04/01 10:36:44 ngoto Exp $
 #
 # = About Bio::PDB
 #
@@ -233,7 +233,7 @@ module Bio
         }
         klass.module_eval {
           symbolary.each do |x|
-            define_method(x) { do_parse; super }
+            define_method(x) { do_parse; super() }
           end
         }
         klass
@@ -267,7 +267,7 @@ module Bio
         }
         klass.module_eval {
           define_method(:initialize_from_string) { |str|
-            r = super
+            r = super(str)
             do_parse
             r
           }
@@ -437,7 +437,8 @@ module Bio
       def self.create_definition_hash
         hash = {}
         constants.each do |x|
-          hash[x] =  const_get(x) if /\A[A-Z][A-Z0-9]+\z/ =~ x
+          x = x.intern # keep compatibility both Ruby 1.8 and 1.9
+          hash[x] = const_get(x) if /\A[A-Z][A-Z0-9]+\z/ =~ x.to_s
         end
         if x = const_get(:Default) then
           hash.default = x
@@ -1380,22 +1381,27 @@ module Bio
       End = 
         def_rec([  2,  1, Pdb_Integer, :serial ]) # dummy field (always 0)
 
-      Definition['END'] = End
+      Definition['END'.intern] = End
 
       # Basically just look up the class in Definition hash
       # do some munging for JRNL and REMARK
       def self.get_record_class(str)
         t = fetch_record_name(str)
+        t = t.intern unless t.empty?
         if d = Definition[t] then
           return d
         end
         case t
-        when 'JRNL'
-          d = Jrnl::Definition[str[12..15].to_s.strip]
-        when 'REMARK'
+        when :JRNL
+          ts = str[12..15].to_s.strip
+          ts = ts.intern unless ts.empty?
+          d = Jrnl::Definition[ts]
+        when :REMARK
           case str[7..9].to_i
           when 1
-            d = Remark1::Definition[str[12..15].to_s.strip]
+            ts = str[12..15].to_s.strip
+            ts = ts.intern unless ts.empty?
+            d = Remark1::Definition[ts]
           when 2
             if str[28..37] == 'ANGSTROMS.' then
               d = Remark2::ANGSTROMS
@@ -1417,13 +1423,21 @@ module Bio
 
     Coordinate_fileds = {
       'MODEL'  => true,
+      :MODEL   => true,
       'ENDMDL' => true,
+      :ENDMDL  => true,
       'ATOM'   => true,
+      :ATOM    => true,
       'HETATM' => true,
+      :HETATM  => true,
       'SIGATM' => true,
+      :SIGATM  => true,
       'SIGUIJ' => true,
+      :SIGUIJ  => true,
       'ANISOU' => true,
+      :ANISOU  => true,
       'TER'    => true,
+      :TER     => true,
     }
 
     # Creates a new Bio::PDB object from given <em>str</em>.
@@ -1483,6 +1497,9 @@ module Bio
               cChain = newChain
               chain = newChain
             end
+            # chain might be changed, clearing cResidue and cLigand
+            cResidue = nil
+            cLigand = nil
           end
         end
 
@@ -1536,6 +1553,8 @@ module Bio
         when 'MODEL'
           c_atom = nil
           cChain = nil
+          cResidue = nil
+          cLigand = nil
           if cModel.model_serial or cModel.chains.size > 0 then
             self.addModel(cModel)
           end
@@ -1795,13 +1814,17 @@ module Bio
               #aa is three letter code: i.e. ALA
               #need to look up with Ala
               aa = aa.capitalize
-              (Bio::AminoAcid.three2one(aa) or 'X')
+              (begin
+                 Bio::AminoAcid.three2one(aa)
+               rescue ArgumentError
+                 nil
+               end || 'X')
             end
             seq = Bio::Sequence::AA.new(a.to_s)
           else
             # nucleic acid sequence
             a.collect! do |na|
-              na = na.strip
+              na = na.delete('^a-zA-Z')
               na.size == 1 ? na : 'n'
             end
             seq = Bio::Sequence::NA.new(a.to_s)
@@ -1876,6 +1899,12 @@ module Bio
       f ? f.modNum : nil
     end
 
+    # returns a string containing human-readable representation
+    # of this object.
+    def inspect
+      "#<#{self.class.to_s} entry_id=#{entry_id.inspect}>"
+    end
+
   end #class PDB
 
 end #module Bio
