@@ -9,7 +9,7 @@
 #               Jan Aerts <jan.aerts@bbsrc.ac.uk>
 # License::     The Ruby License
 #
-# $Id: sequence.rb,v 0.58 2007/04/05 23:35:39 trevor Exp $
+# $Id:$
 #
 
 require 'bio/sequence/compat'
@@ -70,6 +70,10 @@ class Sequence
   autoload :AA,      'bio/sequence/aa'
   autoload :Generic, 'bio/sequence/generic'
   autoload :Format,  'bio/sequence/format'
+  autoload :Adapter, 'bio/sequence/adapter'
+  autoload :QualityScore, 'bio/sequence/quality_score'
+
+  include Format
 
   # Create a new Bio::Sequence object
   #
@@ -95,71 +99,153 @@ class Sequence
   # Pass any unknown method calls to the wrapped sequence object.  see
   # http://www.rubycentral.com/book/ref_c_object.html#Object.method_missing
   def method_missing(sym, *args, &block) #:nodoc:
-    @seq.send(sym, *args, &block)
+    begin
+      seq.__send__(sym, *args, &block)
+    rescue NoMethodError => evar
+      lineno = __LINE__ - 2
+      file = __FILE__
+      bt_here = [ "#{file}:#{lineno}:in \`__send__\'",
+                  "#{file}:#{lineno}:in \`method_missing\'"
+                ]
+      if bt_here == evar.backtrace[0, 2] then
+        bt = evar.backtrace[2..-1]
+        evar = evar.class.new("undefined method \`#{sym.to_s}\' for #{self.inspect}")
+        evar.set_backtrace(bt)
+      end
+      #p lineno
+      #p file
+      #p bt_here
+      #p evar.backtrace
+      raise(evar)
+    end
   end
   
-  # The sequence identifier.  For example, for a sequence
-  # of Genbank origin, this is the accession number.
+  # The sequence identifier (String).  For example, for a sequence
+  # of Genbank origin, this is the locus name.
+  # For a sequence of EMBL origin, this is the primary accession number.
   attr_accessor :entry_id
   
-  # A String with a description of the sequence
+  # A String with a description of the sequence (String)
   attr_accessor :definition
   
-  # An Array of Bio::Feature objects
+  # Features (An Array of Bio::Feature objects)
   attr_accessor :features
   
-  # An Array of Bio::Reference objects
+  # References (An Array of Bio::Reference objects)
   attr_accessor :references
   
-  # A comment String
+  # Comments (String or an Array of String)
   attr_accessor :comments
   
-  # Date from sequence source. Often date of deposition.
-  attr_accessor :date
-  
-  # An Array of Strings
+  # Keywords (An Array of String)
   attr_accessor :keywords
   
-  # An Array of Strings; links to other database entries.
+  # Links to other database entries.
+  # (An Array of Bio::Sequence::DBLink objects)
   attr_accessor :dblinks
-  
-  # A taxonomy String
-  attr_accessor :taxonomy
-  
+
   # Bio::Sequence::NA/AA
   attr_accessor :moltype
   
   # The sequence object, usually Bio::Sequence::NA/AA, 
   # but could be a simple String
   attr_accessor :seq
+
+  # Quality scores of the bases/residues in the sequence.
+  # (Array containing Integer, or nil)
+  attr_accessor :quality_scores
+
+  # The meaning (calculation method) of the quality scores stored in
+  # the <tt>quality_scores</tt> attribute.
+  # Maybe one of :phred, :solexa, or nil.
+  #
+  # Note that if it is nil, and <tt>error_probabilities</tt> is empty,
+  # some methods implicitly assumes that it is :phred (PHRED score).
+  attr_accessor :quality_score_type
+
+  # Error probabilities of the bases/residues in the sequence.
+  # (Array containing Float, or nil)
+  attr_accessor :error_probabilities
+
+  #---
+  # Attributes below have been added during BioHackathon2008
+  #+++
   
-  # Using Bio::Sequence::Format, return a String with the Bio::Sequence
-  # object formatted in the given style.
-  #
-  # Formats currently implemented are: 'fasta', 'genbank', and 'embl'
-  #
-  #   s = Bio::Sequence.new('atgc')
-  #   puts s.output(:fasta)                   #=> "> \natgc\n"
-  #
-  # The style argument is given as a Ruby 
-  # Symbol(http://www.ruby-doc.org/core/classes/Symbol.html)
-  # ---
-  # *Arguments*: 
-  # * (required) _style_: :fasta, :genbank, *or* :embl
-  # *Returns*:: String object
-  def output(style)
-    extend Bio::Sequence::Format
-    case style
-    when :fasta
-      format_fasta
-    when :gff
-      format_gff
-    when :genbank
-      format_genbank
-    when :embl
-      format_embl
-    end
-  end
+  # Version number of the sequence (String or Integer).
+  # Unlike <tt>entry_version</tt>, <tt>sequence_version</tt> will be changed
+  # when the submitter of the sequence updates the entry.
+  # Normally, the same entry taken from different databases (EMBL, GenBank,
+  # and DDBJ) may have the same sequence_version.
+  attr_accessor :sequence_version
+
+  # Topology (String). "circular", "linear", or nil.
+  attr_accessor :topology
+
+  # Strandedness (String). "single" (single-stranded),
+  # "double" (double-stranded), "mixed" (mixed-stranded), or nil.
+  attr_accessor :strandedness
+
+  # molecular type (String). "DNA" or "RNA" for nucleotide sequence.
+  attr_accessor :molecule_type
+
+  # Data Class defined by EMBL (String)
+  # See http://www.ebi.ac.uk/embl/Documentation/User_manual/usrman.html#3_1
+  attr_accessor :data_class
+
+  # Taxonomic Division defined by EMBL/GenBank/DDBJ (String)
+  # See http://www.ebi.ac.uk/embl/Documentation/User_manual/usrman.html#3_2
+  attr_accessor :division
+
+  # Primary accession number (String)
+  attr_accessor :primary_accession
+
+  # Secondary accession numbers (Array of String)
+  attr_accessor :secondary_accessions
+
+  # Created date of the sequence entry (Date, DateTime, Time, or String)
+  attr_accessor :date_created
+
+  # Last modified date of the sequence entry (Date, DateTime, Time, or String)
+  attr_accessor :date_modified
+
+  # Release information when created (String)
+  attr_accessor :release_created
+
+  # Release information when last-modified (String)
+  attr_accessor :release_modified
+
+  # Version of the entry (String or Integer).
+  # Unlike <tt>sequence_version</tt>, <tt>entry_version</tt> is a database
+  # maintainer's internal version number.
+  # The version number will be changed when the database maintainer
+  # modifies the entry.
+  # The same enrty in EMBL, GenBank, and DDBJ may have different
+  # entry_version.
+  attr_accessor :entry_version
+
+  # Organism species (String). For example, "Escherichia coli".
+  attr_accessor :species
+
+  # Organism classification, taxonomic classification of the source organism.
+  # (Array of String)
+  attr_accessor :classification
+  alias taxonomy classification
+
+  # (not well supported) Organelle information (String).
+  attr_accessor :organelle
+
+  # Namespace of the sequence IDs described in entry_id, primary_accession,
+  # and secondary_accessions methods (String).
+  # For example, 'EMBL', 'GenBank', 'DDBJ', 'RefSeq'.
+  attr_accessor :id_namespace
+
+  # Sequence identifiers which are not described in entry_id,
+  # primary_accession,and secondary_accessions methods
+  # (Array of Bio::Sequence::DBLink objects).
+  # For example, NCBI GI number can be stored.
+  # Note that only identifiers of the entry itself should be stored.
+  # For database cross references, <tt>dblinks</tt> should be used.
+  attr_accessor :other_seqids
 
   # Guess the type of sequence, Amino Acid or Nucleic Acid, and create a 
   # new sequence object (Bio::Sequence::AA or Bio::Sequence::NA) on the basis
@@ -174,9 +260,9 @@ class Sequence
   def auto
     @moltype = guess
     if @moltype == NA
-      @seq = NA.new(@seq)
+      @seq = NA.new(seq)
     else
-      @seq = AA.new(@seq)
+      @seq = AA.new(seq)
     end
   end
 
@@ -236,7 +322,7 @@ class Sequence
   # * (optional) _index_: Fixnum (default 1)
   # *Returns*:: Bio::Sequence::NA/AA
   def guess(threshold = 0.9, length = 10000, index = 0)
-    str = @seq.to_s[index,length].to_s.extend Bio::Sequence::Common
+    str = seq.to_s[index,length].to_s.extend Bio::Sequence::Common
     cmp = str.composition
 
     bases = cmp['A'] + cmp['T'] + cmp['G'] + cmp['C'] + cmp['U'] +
@@ -309,7 +395,7 @@ class Sequence
   # ---
   # *Returns*:: Bio::Sequence::NA
   def na
-    @seq = NA.new(@seq)
+    @seq = NA.new(seq)
     @moltype = NA
   end
 
@@ -330,146 +416,58 @@ class Sequence
   # ---
   # *Returns*:: Bio::Sequence::AA
   def aa
-    @seq = AA.new(@seq)
+    @seq = AA.new(seq)
     @moltype = AA
   end
-  
+
+  # Create a new Bio::Sequence object from a formatted string
+  # (GenBank, EMBL, fasta format, etc.)
+  #
+  #   s = Bio::Sequence.input(str)
+  # ---
+  # *Arguments*:
+  # * (required) _str_: string
+  # * (optional) _format_: format specification (class or nil)
+  # *Returns*:: Bio::Sequence object
+  def self.input(str, format = nil)
+    if format then
+      klass = format
+    else
+      klass = Bio::FlatFile::AutoDetect.default.autodetect(str)
+    end
+    obj = klass.new(str)
+    obj.to_biosequence
+  end
+
+  # alias of Bio::Sequence.input
+  def self.read(str, format = nil)
+    input(str, format)
+  end
+
+  # accession numbers of the sequence
+  #
+  # *Returns*:: Array of String
+  def accessions
+    [ primary_accession, secondary_accessions ].flatten.compact
+  end
+
+  # Normally, users should not call this method directly.
+  # Use Bio::*#to_biosequence (e.g. Bio::GenBank#to_biosequence).
+  #
+  # Creates a new Bio::Sequence object from database data with an
+  # adapter module.
+  def self.adapter(source_data, adapter_module)
+    biosequence = self.new(nil)
+    biosequence.instance_eval {
+      remove_instance_variable(:@seq)
+      @source_data = source_data
+    }
+    biosequence.extend(adapter_module)
+    biosequence
+  end
+
 end # Sequence
 
 
 end # Bio
 
-
-if __FILE__ == $0
-
-  puts "== Test Bio::Sequence::NA.new"
-  p Bio::Sequence::NA.new('')
-  p na = Bio::Sequence::NA.new('atgcatgcATGCATGCAAAA')
-  p rna = Bio::Sequence::NA.new('augcaugcaugcaugcaaaa')
-
-  puts "\n== Test Bio::Sequence::AA.new"
-  p Bio::Sequence::AA.new('')
-  p aa = Bio::Sequence::AA.new('ACDEFGHIKLMNPQRSTVWYU')
-
-  puts "\n== Test Bio::Sequence#to_s"
-  p na.to_s
-  p aa.to_s
-
-  puts "\n== Test Bio::Sequence#subseq(2,6)"
-  p na
-  p na.subseq(2,6)
-
-  puts "\n== Test Bio::Sequence#[2,6]"
-  p na
-  p na[2,6]
-
-  puts "\n== Test Bio::Sequence#to_fasta('hoge', 8)"
-  puts na.to_fasta('hoge', 8)
-
-  puts "\n== Test Bio::Sequence#window_search(15)"
-  p na
-  na.window_search(15) {|x| p x}
-
-  puts "\n== Test Bio::Sequence#total({'a'=>0.1,'t'=>0.2,'g'=>0.3,'c'=>0.4})"
-  p na.total({'a'=>0.1,'t'=>0.2,'g'=>0.3,'c'=>0.4})
-
-  puts "\n== Test Bio::Sequence#composition"
-  p na
-  p na.composition
-  p rna
-  p rna.composition
-
-  puts "\n== Test Bio::Sequence::NA#splicing('complement(join(1..5,16..20))')"
-  p na
-  p na.splicing("complement(join(1..5,16..20))")
-  p rna
-  p rna.splicing("complement(join(1..5,16..20))")
-
-  puts "\n== Test Bio::Sequence::NA#complement"
-  p na.complement
-  p rna.complement
-  p Bio::Sequence::NA.new('tacgyrkmhdbvswn').complement
-  p Bio::Sequence::NA.new('uacgyrkmhdbvswn').complement
-
-  puts "\n== Test Bio::Sequence::NA#translate"
-  p na
-  p na.translate
-  p rna
-  p rna.translate
-
-  puts "\n== Test Bio::Sequence::NA#gc_percent"
-  p na.gc_percent
-  p rna.gc_percent
-
-  puts "\n== Test Bio::Sequence::NA#illegal_bases"
-  p na.illegal_bases
-  p Bio::Sequence::NA.new('tacgyrkmhdbvswn').illegal_bases
-  p Bio::Sequence::NA.new('abcdefghijklmnopqrstuvwxyz-!%#$@').illegal_bases
-
-  puts "\n== Test Bio::Sequence::NA#molecular_weight"
-  p na
-  p na.molecular_weight
-  p rna
-  p rna.molecular_weight
-
-  puts "\n== Test Bio::Sequence::NA#to_re"
-  p Bio::Sequence::NA.new('atgcrymkdhvbswn')
-  p Bio::Sequence::NA.new('atgcrymkdhvbswn').to_re
-  p Bio::Sequence::NA.new('augcrymkdhvbswn')
-  p Bio::Sequence::NA.new('augcrymkdhvbswn').to_re
-
-  puts "\n== Test Bio::Sequence::NA#names"
-  p na.names
-
-  puts "\n== Test Bio::Sequence::NA#pikachu"
-  p na.pikachu
-
-  puts "\n== Test Bio::Sequence::NA#randomize"
-  print "Orig  : "; p na
-  print "Rand  : "; p na.randomize
-  print "Rand  : "; p na.randomize
-  print "Rand  : "; p na.randomize.randomize
-  print "Block : "; na.randomize do |x| print x end; puts
-
-  print "Orig  : "; p rna
-  print "Rand  : "; p rna.randomize
-  print "Rand  : "; p rna.randomize
-  print "Rand  : "; p rna.randomize.randomize
-  print "Block : "; rna.randomize do |x| print x end; puts
-
-  puts "\n== Test Bio::Sequence::NA.randomize(counts)"
-  print "Count : "; p counts = {'a'=>10,'c'=>20,'g'=>30,'t'=>40}
-  print "Rand  : "; p Bio::Sequence::NA.randomize(counts)
-  print "Count : "; p counts = {'a'=>10,'c'=>20,'g'=>30,'u'=>40}
-  print "Rand  : "; p Bio::Sequence::NA.randomize(counts)
-  print "Block : "; Bio::Sequence::NA.randomize(counts) {|x| print x}; puts
-
-  puts "\n== Test Bio::Sequence::AA#codes"
-  p aa
-  p aa.codes
-
-  puts "\n== Test Bio::Sequence::AA#names"
-  p aa
-  p aa.names
-
-  puts "\n== Test Bio::Sequence::AA#molecular_weight"
-  p aa.subseq(1,20)
-  p aa.subseq(1,20).molecular_weight
-
-  puts "\n== Test Bio::Sequence::AA#randomize"
-  aaseq = 'MRVLKFGGTSVANAERFLRVADILESNARQGQVATVLSAPAKITNHLVAMIEKTISGQDA'
-  s = Bio::Sequence::AA.new(aaseq)
-  print "Orig  : "; p s
-  print "Rand  : "; p s.randomize
-  print "Rand  : "; p s.randomize
-  print "Rand  : "; p s.randomize.randomize
-  print "Block : "; s.randomize {|x| print x}; puts
-
-  puts "\n== Test Bio::Sequence::AA.randomize(counts)"
-  print "Count : "; p counts = s.composition
-  print "Rand  : "; puts Bio::Sequence::AA.randomize(counts)
-  print "Block : "; Bio::Sequence::AA.randomize(counts) {|x| print x}; puts
-
-end
-
-
