@@ -4,14 +4,15 @@
 # Copyright::  Copyright (C) 2005 Mitsuteru Nakao <n@bioruby.org>
 # License::    The Ruby License
 #
-#  $Id: test_embl.rb,v 1.5 2007/04/05 23:35:43 trevor Exp $
+#  $Id:$
 #
 
+# loading helper routine for testing bioruby
 require 'pathname'
-libpath = Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 5, 'lib')).cleanpath.to_s
-$:.unshift(libpath) unless $:.include?(libpath)
-
+load Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 4,
+                            'bioruby_test_helper.rb')).cleanpath.to_s
 
+# libraries needed for the tests
 require 'test/unit'
 require 'bio/db/embl/embl'
 
@@ -19,8 +20,7 @@ module Bio
   class TestEMBL < Test::Unit::TestCase
     
     def setup
-    bioruby_root = Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 5)).cleanpath.to_s
-      output = File.open(File.join(bioruby_root, 'test', 'data', 'embl', 'AB090716.embl')).read
+      output = File.read(File.join(BioRubyTestDataPath, 'embl', 'AB090716.embl'))
       @obj = Bio::EMBL.new(output)
     end
 
@@ -150,7 +150,7 @@ module Bio
 
     # Bio::EMBLDB::COMMON#references
     def test_references 
-      assert_equal(Bio::References, @obj.references.class)
+      assert_equal(Array, @obj.references.class)
     end
 
     # Bio::EMBLDB::COMMON#dr
@@ -163,7 +163,7 @@ module Bio
     end
 
     def test_ft
-      assert_equal(Bio::Features, @obj.ft.class)
+      assert_equal(Array, @obj.ft.class)
     end
 
     def test_ft_iterator
@@ -173,7 +173,7 @@ module Bio
     end
 
     def test_ft_accessor
-      assert_equal('CDS', @obj.ft.features[1].feature)
+      assert_equal('CDS', @obj.ft[1].feature)
     end
 
     def test_each_cds
