@@ -6,13 +6,15 @@
 #              2006 Mitsuteru C. Nakao <n@bioruby.org>
 # License::    The Ruby License
 #
-#  $Id: test_sequence.rb,v 1.9 2007/04/05 23:35:42 trevor Exp $
+#  $Id:$
 #
 
+# loading helper routine for testing bioruby
 require 'pathname'
-libpath = Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 3, 'lib')).cleanpath.to_s
-$:.unshift(libpath) unless $:.include?(libpath)
+load Pathname.new(File.join(File.dirname(__FILE__), ['..'] * 2,
+                            'bioruby_test_helper.rb')).cleanpath.to_s
 
+# libraries needed for the tests
 require 'test/unit'
 require 'bio/sequence'
 
@@ -321,7 +323,7 @@ module Bio
     def test_translate_5
       assert_equal("", @obj.translate(5))
     end
-    def test_translate_5
+    def test_translate_6
       assert_equal("", @obj.translate(6))
     end
   end
